@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2018 the Babel team
 # All rights reserved.
 #
-# This software is licensed as described in the file COPYING, which
+# This software is licensed as described in the file LICENSE, which
 # you should have received as part of this distribution. The terms
 # are also available at http://babel.edgewall.org/wiki/License.
 #
@@ -13,6 +13,7 @@
 
 from datetime import datetime
 import unittest
+import sys
 
 from babel.core import Locale
 from babel.messages.catalog import Catalog, Message
@@ -20,7 +21,6 @@ from babel.messages import pofile
 from babel.util import FixedOffsetTimezone
 from babel._compat import StringIO, BytesIO
 
-
 class ReadPoTestCase(unittest.TestCase):
 
     def test_preserve_locale(self):
@@ -36,6 +36,12 @@ msgstr ""
 "Language: en_US\n"''')
         catalog = pofile.read_po(buf, locale='de')
         self.assertEqual(Locale('en', 'US'), catalog.locale)
+        buf = StringIO(r'''
+msgid ""
+msgstr ""
+"Language: ko-KR\n"''')
+        catalog = pofile.read_po(buf, locale='de')
+        self.assertEqual(Locale('ko', 'KR'), catalog.locale)
 
     def test_preserve_domain(self):
         buf = StringIO(r'''msgid "foo"
@@ -423,6 +429,71 @@ msgstr[2] "Vohs [text]"
         self.assertEqual("", message.string[1])
         self.assertEqual("Vohs [text]", message.string[2])
 
+    def test_abort_invalid_po_file(self):
+        invalid_po = '''
+            msgctxt ""
+            "{\"checksum\": 2148532640, \"cxt\": \"collector_thankyou\", \"id\": "
+            "270005359}"
+            msgid ""
+            "Thank you very much for your time.\n"
+            "If you have any questions regarding this survey, please contact Fulano "
+            "at nadie@blah.com"
+            msgstr "Merci de prendre le temps de remplir le sondage.
+            Pour toute question, veuillez communiquer avec Fulano  à nadie@blah.com
+            "
+        '''
+        invalid_po_2 = '''
+            msgctxt ""
+            "{\"checksum\": 2148532640, \"cxt\": \"collector_thankyou\", \"id\": "
+            "270005359}"
+            msgid ""
+            "Thank you very much for your time.\n"
+            "If you have any questions regarding this survey, please contact Fulano "
+            "at fulano@blah.com."
+            msgstr "Merci de prendre le temps de remplir le sondage.
+            Pour toute question, veuillez communiquer avec Fulano a fulano@blah.com
+            "
+            '''
+        # Catalog not created, throws Unicode Error
+        buf = StringIO(invalid_po)
+        output = None
+
+        # This should only be thrown under py27
+        if sys.version_info.major == 2:
+            with self.assertRaises(UnicodeEncodeError):
+                output = pofile.read_po(buf, locale='fr', abort_invalid=False)
+            assert not output
+        else:
+            output = pofile.read_po(buf, locale='fr', abort_invalid=False)
+            assert isinstance(output, Catalog)
+
+        # Catalog not created, throws PoFileError
+        buf = StringIO(invalid_po_2)
+        output = None
+        with self.assertRaises(pofile.PoFileError) as e:
+            output = pofile.read_po(buf, locale='fr', abort_invalid=True)
+        assert not output
+
+        # Catalog is created with warning, no abort
+        buf = StringIO(invalid_po_2)
+        output = pofile.read_po(buf, locale='fr', abort_invalid=False)
+        assert isinstance(output, Catalog)
+
+        # Catalog not created, aborted with PoFileError
+        buf = StringIO(invalid_po_2)
+        output = None
+        with self.assertRaises(pofile.PoFileError) as e:
+            output = pofile.read_po(buf, locale='fr', abort_invalid=True)
+        assert not output
+
+    def test_invalid_pofile_with_abort_flag(self):
+        parser = pofile.PoFileParser(None, abort_invalid=True)
+        lineno = 10
+        line = 'Algo esta mal'
+        msg = 'invalid file'
+        with self.assertRaises(pofile.PoFileError) as e:
+            parser._invalid_pofile(line, lineno, msg)
+
 
 class WritePoTestCase(unittest.TestCase):
 
@@ -680,6 +751,41 @@ msgstr[0] "Voh"
 msgstr[1] "Voeh"''' in value
         assert value.find(b'msgid ""') < value.find(b'msgid "bar"') < value.find(b'msgid "foo"')
 
+    def test_sorted_po_context(self):
+        catalog = Catalog()
+        catalog.add((u'foo', u'foos'), (u'Voh', u'Voeh'),
+                    locations=[('main.py', 1)],
+                    context='there')
+        catalog.add((u'foo', u'foos'), (u'Voh', u'Voeh'),
+                    locations=[('main.py', 1)])
+        catalog.add((u'foo', u'foos'), (u'Voh', u'Voeh'),
+                    locations=[('main.py', 1)],
+                    context='here')
+        buf = BytesIO()
+        pofile.write_po(buf, catalog, sort_output=True)
+        value = buf.getvalue().strip()
+        # We expect the foo without ctx, followed by "here" foo and "there" foo
+        assert b'''\
+#: main.py:1
+msgid "foo"
+msgid_plural "foos"
+msgstr[0] "Voh"
+msgstr[1] "Voeh"
+
+#: main.py:1
+msgctxt "here"
+msgid "foo"
+msgid_plural "foos"
+msgstr[0] "Voh"
+msgstr[1] "Voeh"
+
+#: main.py:1
+msgctxt "there"
+msgid "foo"
+msgid_plural "foos"
+msgstr[0] "Voh"
+msgstr[1] "Voeh"''' in value
+
     def test_file_sorted_po(self):
         catalog = Catalog()
         catalog.add(u'bar', locations=[('utils.py', 3)])
@@ -759,3 +865,25 @@ class PofileFunctionsTestCase(unittest.TestCase):
         self.assertEqual(expected_denormalized, pofile.denormalize(msgstr))
         self.assertEqual(expected_denormalized,
                          pofile.denormalize('""\n' + msgstr))
+
+
+def test_unknown_language_roundtrip():
+    buf = StringIO(r'''
+msgid ""
+msgstr ""
+"Language: sr_SP\n"''')
+    catalog = pofile.read_po(buf)
+    assert catalog.locale_identifier == 'sr_SP'
+    assert not catalog.locale
+    buf = BytesIO()
+    pofile.write_po(buf, catalog)
+    assert 'sr_SP' in buf.getvalue().decode()
+
+
+def test_unknown_language_write():
+    catalog = Catalog(locale='sr_SP')
+    assert catalog.locale_identifier == 'sr_SP'
+    assert not catalog.locale
+    buf = BytesIO()
+    pofile.write_po(buf, catalog)
+    assert 'sr_SP' in buf.getvalue().decode()
