@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2018 the Babel team
 # All rights reserved.
 #
-# This software is licensed as described in the file COPYING, which
+# This software is licensed as described in the file LICENSE, which
 # you should have received as part of this distribution. The terms
 # are also available at http://babel.edgewall.org/wiki/License.
 #
@@ -15,8 +15,7 @@ import unittest
 import random
 from operator import methodcaller
 
-from babel import localedata
-
+from babel import localedata, numbers
 
 class MergeResolveTestCase(unittest.TestCase):
 
@@ -79,7 +78,6 @@ def test_locale_identification():
     for l in localedata.locale_identifiers():
         assert localedata.exists(l)
 
-
 def test_unique_ids():
     # Check all locale IDs are uniques.
     all_ids = localedata.locale_identifiers()
@@ -94,3 +92,16 @@ def test_mixedcased_locale():
         locale_id = ''.join([
             methodcaller(random.choice(['lower', 'upper']))(c) for c in l])
         assert localedata.exists(locale_id)
+
+def test_locale_argument_acceptance():
+    # Testing None input.
+    normalized_locale = localedata.normalize_locale(None)
+    assert normalized_locale is None
+    locale_exist = localedata.exists(None)
+    assert locale_exist == False
+
+    # # Testing list input.
+    normalized_locale = localedata.normalize_locale(['en_us', None])
+    assert normalized_locale is None
+    locale_exist = localedata.exists(['en_us', None])
+    assert locale_exist == False
